(function(global, factory) {
	'use strict';

	if (typeof exports === 'object') {
		module.exports = factory(require('angular'));
	} else if (typeof define === 'function' && define.amd) {
		define(['angular'], factory);
	} else {
		factory(global.angular);
	}

}(window, function(angular) {
	'use strict';

	var checkMandatory = function(prop, desc) {
		if (!prop) {
			throw new Error(desc);
		}
	};

	var defaults = function(dest, src) {
		for (var key in src) {
			if (typeof dest[key] === 'undefined') {
				dest[key] = src[key];
			}
		}
	};

	var baiduMapDir = function() {
  
		return {
			restrict: 'E',
			scope: {
				'options': '='
			},
			link: function($scope, element, attrs) {

				var defaultOpts = {
					navCtrl: false,
					scaleCtrl: false,
					overviewCtrl: false,
					enableScrollWheelZoom: false,
					setCurrentCity: false,
					zoom: 10
				};

				var opts = $scope.options;
				defaults(opts, defaultOpts);
				checkMandatory(opts.center, 'options.center must be set');
				checkMandatory(opts.center.longitude, 'options.center.longitude must be set');
				checkMandatory(opts.center.latitude, 'options.center.latitude must be set');
				checkMandatory(opts.city, 'options.city must be set');

				var map = new BMap.Map(element.find('div')[0]);
				if (opts.navCtrl) {
					map.addControl(new BMap.NavigationControl());
				}
				if (opts.scaleCtrl) {
					map.addControl(new BMap.ScaleControl());
				}
				if (opts.overviewCtrl) {
					map.addControl(new BMap.OverviewMapControl());
				}
				if (opts.enableScrollWheelZoom) {
					map.enableScrollWheelZoom();
				}
				if (opts.setCurrentCity) {
					map.setCurrentCity(opts.city);
				}
				if (!opts.markers) {
					return;
				}
				var previousMarkers = [];

				var openInfoWindow = function(infoWin) {
					return function() {
						this.openInfoWindow(infoWin);
					};
				};
				var baiduPoint;
				var mark = function() {
					var i = 0;
					for (i = 0; i < previousMarkers.length; i++) {
						previousMarkers[i].removeEventListener('click', openInfoWindow(infoWindow2));
						map.removeOverlay(previousMarkers[i]);
					}
					previousMarkers.length = 0;
					for (i = 0; i < opts.markers.length; i++) {
						var marker = opts.markers[i];
						var pt = new BMap.Point(marker.longitude, marker.latitude);
						var getCityByBaiduCoordinate = function(rs) {
							var baiduAddress = rs.addressComponents;
							var address = baiduAddress.province + baiduAddress.city + baiduAddress.district + baiduAddress.street + baiduAddress.streetNumber;
							var ppt = new BMap.Point(baiduPoint.lng, baiduPoint.lat);
							map.centerAndZoom(ppt, opts.zoom);
							var marker2 = new BMap.Marker(ppt);
							map.addOverlay(marker2);
							previousMarkers.push(marker2);
							if (!marker.title && !marker.content) {
								return;
							}
							var infoWindow2 = new BMap.InfoWindow('<p>' + (marker.title ? marker.title : '') + '</p><p>' + (marker.content ? marker.content : address) + '</p>', {
								enableMessage: !!marker.enableMessage
							});
							marker2.addEventListener('click', openInfoWindow(infoWindow2));
						}
						if(opts.isBaiduPoint) {
							baiduPoint = pt;
							var geoc = new BMap.Geocoder();
							geoc.getLocation(baiduPoint, getCityByBaiduCoordinate);
						} else {
							var translateCallback = function(point) {
								baiduPoint = point;
								var geoc = new BMap.Geocoder();
								geoc.getLocation(baiduPoint, getCityByBaiduCoordinate);
							}
							BMap.Convertor.translate(pt, 0, translateCallback);
						}
					}
				};
				mark();
				$scope.$watch('options.center', function(newValue, oldValue) {
					opts = $scope.options;
					mark();
				}, true);

				$scope.$watch('options.markers', function(newValue, oldValue) {
					mark();
				}, true);
			},
			template: '<div style="width: 100%; height: 100%;"></div>'
		};
	};
	var baiduMap = angular.module('baiduMap', []);
	baiduMap.directive('baiduMap', [baiduMapDir]);
}));